
import pygame
import pygame.mixer
from pygame.locals import *
from pygame.sprite import Sprite
import pygame,pygame.font
import sound

import random
from math import *

import menu
from util import Util, render_text, fade_to_black
import data, os
from locals import *

class Clock(pygame.sprite.Sprite):
    """moves a monkey critter across the screen. it can spin the
       monkey when it is punched."""
    def __init__(self,countdown=15):
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        self.countdown = countdown
        self.remaining = countdown
        self.start_time = pygame.time.get_ticks()        
        self.image = render_text("Time: "+str(self.remaining), (0,0,0), (255,255,255))
        self.rect = self.image.get_rect(left=0,top=0)
        

    def update(self):
        ticks = (pygame.time.get_ticks() - self.start_time)
        self.remaining = (self.countdown - (ticks / 1000) % 60)
        self.image = render_text("Time: "+str(self.remaining), (0,0,0), (255,255,255))

class Score(pygame.sprite.Sprite):
    """moves a monkey critter across the screen. it can spin the
       monkey when it is punched."""
    def __init__(self,killed=0):
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        self.killed = killed     
        self.image = render_text("Slain: "+str(self.killed), (0,0,0), (255,255,255))
        self.rect = self.image.get_rect(right=SCREEN_WIDTH,top=0)
        
    def get_score(self):
        return self.killed

    def update(self):
        self.image = render_text("Slain: "+str(self.killed), (0,0,0), (255,255,255))
        self.rect = self.image.get_rect(right=SCREEN_WIDTH,top=0)
        
    def update_score(self, dead=1):
        self.killed+=dead

class Hamster(pygame.sprite.Sprite):
    def __init__(self,x,y,level=1):
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        self.image = data.load_image(os.path.join("hamster", "standing.png"),True)
        self.image2 = data.load_image(os.path.join("hamster", "standing1_5.png"),True)
        self.rect = self.image.get_rect(center=(x,y))
        self.level = level
        self.smashed = False
        self.x = random.randint(-level,level)
        self.y = random.randint(-level,level)

    def level_up(self, level=1):
        self.level += level

    def update(self):
        if self.smashed:
            return
        level = self.level
        offset = level % 4
        time = pygame.time.get_ticks()
        
        if offset==0:
            self.rect.move_ip(self.x*sin(time * 0.002),self.y*cos(time * 0.002))
        if offset==1:
            self.rect.move_ip(random.randint(-level,level),random.randint(-level,level))
        elif offset==2:
            if self.rect.top <= 0 or self.rect.bottom >= SCREEN_HEIGHT:
                self.y = -self.y
            if (self.rect.left <= 0) or (self.rect.right >= SCREEN_WIDTH):
                self.x = -self.x
            self.rect.move_ip(self.x, self.y)
        elif offset==3:
            x,y = pygame.mouse.get_pos()
            if x > self.rect.centerx:
                self.rect.move_ip(-level,0)
            else:
                self.rect.move_ip(level,0)        
                
            if y > self.rect.centery:
                self.rect.move_ip(0,-level)
            else:
                self.rect.move_ip(0,level)
            self.rect.move_ip(random.randint(int(-level/2),int(level/2)),random.randint(int(-level/2),int(level/2)))
        else:
            self.rect.move_ip(random.randint(-level,level),random.randint(-level,level))
            
        self.rect.clamp_ip(Rect(0,0,SCREEN_WIDTH,SCREEN_HEIGHT))

    def smash(self):
        self.smashed = True
        self.image = self.image2
        self.rect = self.image.get_rect(center=self.rect.center,bottom=self.rect.bottom)
        #self.rect2.top = self.rect.bottom - self.rect.height
        #self.rect = self.rect2


class Hammer(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        self.image1 = data.load_image(os.path.join("hamster", "hammer_1.png"),True) 
        #self.image1 = data.load_image(os.path.join("hamster", "hammer1.png"),True) 
        self.image2 = data.load_image(os.path.join("hamster", "hammer_2.png"),True)
        self.image = self.image1
        self.rect = self.image.get_rect() # 365, 370
        self.head_rect = pygame.Rect((self.rect.center),(30,8))
        self.hit = False

    def move(self,pos):
        x,y = pos #center = 183,185
        #self.rect.center = pos
        self.rect.center = (x-143,y-39)        
        self.head_rect.center = (x, y+50)
        if self.hit:
            self.rect.move_ip(0, 50)

        
    def h(self):
        self.hit = True
        self.image = self.image2
        
    def uh(self):
        self.hit = False
        self.image = self.image1

class TempSprite(pygame.sprite.Sprite):
    def __init__(self,rect):
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        self.image = None
        self.rect = rect

def play(screen):
    #Load sounds
    sound.play_music("hamster_dance")
     
    level = 1
    full_score = 0
    done = False 
    while not done:        
        score, hams = do_level(screen, level, full_score)
        if score != hams:
            done = True
        else:
            full_score += score
            c = Clock(15)
            level += 1

    #Fade Out Music
    sound.stop_music()
    return full_score

    
def do_level(screen, level, full_score):
    #Setup Screen
    background = pygame.Surface(screen.get_size())
    background = background.convert()
    background.fill((250, 250, 250))
    screen.blit(background, (0, 0))
    pygame.display.flip()
    
    #Create Groups and Sprites
    all = pygame.sprite.OrderedUpdates()

    hamsters = pygame.sprite.OrderedUpdates()
    ham = Hamster(0,0)
    for y in range (ham.rect.height,SCREEN_HEIGHT,int(ham.rect.height*1.5)):
        for x in range (ham.rect.width,SCREEN_WIDTH,int(ham.rect.width*1.5)):
            hamsters.add(Hamster(x,y,level))
    all.add(hamsters)
    
    h = Hammer()
    all.add(h)
    
    c = Clock(15)
    all.add(c)
    
    oldscore = full_score
    s = Score(full_score)
    all.add(s)

    #Other Variables    
    killed = 0
    random.seed()
    clock = pygame.time.Clock()
    hit = None

    #Main game loop 
    done = False
    while not done: 
        #Time keeps on slipping, slipping, slipping; into the future        
        if c.remaining <= 0:
            done = True
            break
        if len(hamsters) == (s.get_score()-oldscore):
            done = True
            break
    
        #Handle player input
        for event in pygame.event.get():
            menu.universal_keys(event,screen) 
                
            if (event.type == KEYDOWN and event.key == K_ESCAPE):
                done = True        
            if event.type == QUIT:
                done = True
            if event.type == MOUSEBUTTONDOWN and event.button == 1:
                h.h()
                hit = TempSprite(h.head_rect)
                #Generate a small area sprite that we can use for hit detection in pygame.sprite.spritecollide
            if event.type == MOUSEBUTTONUP and event.button == 1:
                h.uh()

        #Clear the background over each sprites current location, erasing it.
        all.clear(screen, background) 
    
        #Move/Updates the sprites
        all.update()
        h.move(pygame.mouse.get_pos())

        #Collisions
        if hit:
            for ham in pygame.sprite.spritecollide(hit, hamsters, 0):
                if not ham.smashed:
                    s.update_score()
                    ham.smash()
                    sound.play_sound("squish")
        hit = None
        
        #Append the sprite's new location to my dirty_rects list.
        dirty = all.draw(screen)        
        #Call display.update(dirty_rects)
        pygame.display.update(dirty) 
        
        #Game Delay
        clock.tick(60) 
    #End while
    
    #Return Score
    #print "You killed",killed,"hamsters!"
    return (s.get_score()-oldscore),len(hamsters)
    
#End play